import argparse
import yaml
import base64
import json
import sys
import os
import subprocess

def parse_arguments():
    parser = argparse.ArgumentParser(description='Generate Kubernetes Job and Service YAML.')
    parser.add_argument('--kubeconfig', required=True, default='default', help='Kubernetes config file')
    parser.add_argument('--namespace', required=True, default='default', help='Kubernetes namespace')
    parser.add_argument('--vks_id', required=True, help='vks_id')
    parser.add_argument('--pvc_name', required=True, help='Name of the PersistentVolumeClaim.')
    parser.add_argument('--work_dir', required=True, help='Name of the workdir.')
    parser.add_argument('--cmd', required=True, help='cmd.')
    parser.add_argument('--docker_server', required=True, help='website of image repository.')
    parser.add_argument('--docker_username', required=True, help='userName of the image repository.')
    parser.add_argument('--docker_password', required=True, help='password of the image repository.')
    parser.add_argument('--email', required=True)
    parser.add_argument('--labels', required=True, default='{"app": "distributed-training"}', help='Labels for the Job and Service')
    parser.add_argument('--image', required=True, default='example-image:latest', help='Container image to use')
    parser.add_argument('--harbor_secret', required=True, default='harbor-secret', help='Secret for pull image')
    parser.add_argument('--memory', default='4Gi', help='Memory resource limit for the container')
    parser.add_argument('--cpu', default='1', help='CPU resource limit for the container')
    parser.add_argument('--gpu_type', required=True, default='nvidia.com/gpu', help='GPU type resource name')
    parser.add_argument('--gpu_count', default=1, type=int, help='Number of GPUs to request')
    # parser.add_argument('--master_addr', required=True, default='localhost', help='Master address for distributed training')
    parser.add_argument('--master_port', default=12345, type=int, help='Master port for distributed training')
    parser.add_argument('--nnodes', default=1, type=int, help='Number of nodes for distributed training')
    parser.add_argument('--nproc_per_node', default=1, type=int, help='Number of processes per node')
    parser.add_argument('--service_name', required=True, help='service_name for distributed training')
    
    return parser.parse_args()



def create_secret_yaml(args):
    # 创建 Docker config JSON
    docker_config = {
        "auths": {
            args.docker_server: {
                "username": args.docker_username,
                "password": args.docker_password,
                "email": args.email
            }
        }
    }
    # print('docker_config:',docker_config)

    docker_config_json = json.dumps(docker_config)
    # print('docker_config_json:',docker_config_json)
    
    # 对 Docker config JSON 进行 base64 编码
    docker_config_base64 = base64.b64encode(docker_config_json.encode()).decode('utf-8')
    # print('docker_config_base64:',docker_config_base64)

    secret_yaml = {
        'apiVersion': 'v1',
        'kind': 'Secret',
        'metadata': {
            'name': 'harbor-secret',
            'namespace': args.namespace,
        },
        'type': 'kubernetes.io/dockerconfigjson',
        'data': {
            '.dockerconfigjson': docker_config_base64
        }
    }

    return secret_yaml

def generate_job_yaml(args):
    job_yaml = {
        'apiVersion': 'batch/v1',
        'kind': 'Job',
        'metadata': {
            'name': 'distributed-training-job',
            'namespace': args.namespace,
            # 'labels': yaml.safe_load(args.labels)
            'labels':{
                'app': args.labels
            }
        },
        'spec': {
            'parallelism': args.nnodes,
            'completions': args.nnodes,
            'completionMode': 'Indexed',
            'template': {
                'metadata': {
                    'labels': {
                        'app': args.labels
                    }
                },
                'spec': {
                    'restartPolicy': 'Never',
                    'containers': [{
                        'name': 'coding-dev-container',
                        'image': args.image,
                        'resources': {
                            'requests': {
                                'memory': args.memory,
                                'cpu': args.cpu,
                                args.gpu_type: args.gpu_count
                            },
                            'limits': {
                                'memory': args.memory,
                                'cpu': args.cpu,
                                args.gpu_type: args.gpu_count
                            }
                        },
                        'command': ["sh", "-c", args.cmd],
                        'ports': [{
                            'containerPort': args.master_port,
                            'name': 'http-port',
                            'protocol': 'TCP'
                        }],
                        'volumeMounts': [
                            {'name': 'workspace', 'mountPath': args.work_dir},
                            {'name': 'dshm', 'mountPath': '/dev/shm'}
                        ],
                        'env': [
                            {'name': 'NCCL_IB_DISABLE', 'value': '0'},
                            {'name': 'NCCL_SOCKET_IFNAME', 'value': 'eth0'},
                            {'name': 'NCCL_IB_HCA', 'value': 'ib7s'},
                            {'name': 'nproc_per_node', 'value': str(args.nproc_per_node)},
                            {'name': 'nnodes', 'value': str(args.nnodes)},
                            {'name': 'node_rank', 'valueFrom': {'fieldRef': {'fieldPath': 'metadata.annotations[\'batch.kubernetes.io/job-completion-index\']'}}},
                            {'name': 'master_addr', 'value': args.master_addr},
                            {'name': 'master_port', 'value': str(args.master_port)},
                            {'name': 'pod_num', 'valueFrom': {'fieldRef': {'fieldPath': 'metadata.annotations[\'batch.kubernetes.io/job-completion-index\']'}}},
                            {'name': 'POD_IP', 'valueFrom': {'fieldRef': {'fieldPath': 'status.podIP'}}}
                        ]
                    }],
                    'imagePullSecrets': [{'name': str(args.harbor_secret)}],
                    'volumes': [
                        {'name': 'workspace', 'persistentVolumeClaim': {'claimName': args.pvc_name}},
                        {'name': 'dshm', 'emptyDir': {'medium': 'Memory', 'sizeLimit': '20G'}}
                    ],
                    'tolerations': [{'key': 'nvidia.com/gpu', 'operator': 'Exists', 'effect': 'NoSchedule'}]
                }
            }
        }
    }
    return job_yaml

def generate_service_yaml(args):
    service_yaml = {
        'apiVersion': 'v1',
        'kind': 'Service',
        'metadata': {
            'name': args.service_name,
            'namespace': args.namespace,
            # 'labels': yaml.safe_load(args.labels)
            'labels':{
                'app': args.labels
            }
        },
        'spec': {
            'clusterIP': 'None',  # Headless service
            'selector': {
                'app': args.labels
            },
            'ports': [
                {
                    'name': 'http-port',
                    'port': args.master_port,
                    'protocol': 'TCP'
                }
            ]
        }
    }
    return service_yaml


def generate_namespace_yaml(args):
    namespace_yaml = {
        'apiVersion': 'v1',
        'kind': 'Namespace',
        'metadata': {
            'name': args.namespace,
            'labels':{
                'app': args.labels
            }
        }
    }
    return namespace_yaml

def generate_pvc_yaml(args):
    pvc_yaml = {
        'apiVersion': 'v1',
        'kind': 'PersistentVolumeClaim',
        'metadata': {
            'name': args.pvc_name,
            'namespace': args.namespace,
        },
        'spec': {
            'accessModes': [
                'ReadWriteMany'
            ],
            'selector': {
                'matchLabels': {
                    'vcluster': args.vks_id
                }
            },
            'resources': {
                'requests': {
                    'storage': '1000Gi'
                }
            },
            'volumeMode': 'Filesystem'
        }
    }
    return pvc_yaml

def write_yaml_to_file(yaml_data, filename):
    with open(filename, 'w') as file:
        yaml.dump(yaml_data, file, default_flow_style=False)

def set_env_variable(var_name, value):
    if sys.platform == "win32":
        # Windows系统使用set命令
        os.system(f'setx {var_name} "{value}"')
    else:
        # Unix-like系统使用export命令
        # os.system(f'export {var_name}="{value}"')

        # 设置环境变量
        print(var_name,value)
        os.environ[var_name] = value

def apply_yaml_file(filename):
    result = subprocess.run(['kubectl', 'apply', '-f', filename], capture_output=True, text=True)
    if result.returncode == 0:
        print(f"Applied {filename} successfully.")
    else:
        print(f"Failed to apply {filename}. Error: {result.stderr}")

def main():
    args = parse_arguments()
    args.master_addr = args.service_name + "." + args.namespace + ".svc.cluster.local"
    print("args:",args)

    namespace_yaml = generate_namespace_yaml(args)
    pvc_yaml = generate_pvc_yaml(args)
    secret_yaml = create_secret_yaml(args)
    service_yaml = generate_service_yaml(args)
    job_yaml = generate_job_yaml(args)

    write_yaml_to_file(namespace_yaml, 'namespace.yaml')
    write_yaml_to_file(pvc_yaml, 'pvc.yaml')
    write_yaml_to_file(secret_yaml, 'secret.yaml')
    write_yaml_to_file(service_yaml, 'service.yaml')
    write_yaml_to_file(job_yaml, 'job.yaml')

    set_env_variable('KUBECONFIG', args.kubeconfig)

    # 应用YAML文件
    apply_yaml_file('namespace.yaml')
    apply_yaml_file('pvc.yaml')
    apply_yaml_file('secret.yaml')
    apply_yaml_file('service.yaml')
    apply_yaml_file('job.yaml')
    

if __name__ == '__main__':
    main()
